/* aplikacija.cc
 *
 * Copyright (C) 2006 Mirko Stojadinovic
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "aplikacija.h"
#include "rezultat.h"
#include "bcd.h"
#include "visak3.h"
#include "8421.h" 

#include <gtkmm/messagedialog.h>
#include <glibmm/fileutils.h>
#include <iostream>
#include <stdexcept>

Aplikacija::Aplikacija()
{

	#ifdef WIN32
    	static char *cdirname = g_win32_get_package_installation_directory (NULL, NULL);
    	std::string dirname = cdirname; 
	std::cout << "Testing: " << dirname << std::endl;
	#endif

	{  // Glejd opis sucelja
		#ifdef WIN32
    		std::string filename = dirname + "\\kalkulator.glade";//za Windows
		#else  
    		std::string filename = PACKAGE_DATA_DIR "/kalkulator/kalkulator.glade";// gde je na Linuxu instaliran kalkulator.glade
		#endif 

    	if(!Glib::file_test(filename, Glib::FILE_TEST_EXISTS))
        	g_error("Калкулатор: Не могу да пронађем инсталирану Глејд датотеку.\n");

    	try { refGlade = Gnome::Glade::Xml::create(filename); }	// pronadjena je glade datoteka, ucitava se
    	catch(Gnome::Glade::XmlError e)
    	{				// nije uspelo citanje - datoteka nije ispravna
       	std::cerr << e.what() << std::endl;			// sta li se dogodilo?
       	g_error("Калкулатор: Не могу да учитам Глејд датотеку.\n");
    	}

	}
	try
	{ // Ikonica prozora
    	set_default_size(580,-1);
    	set_title("БЦД Калкулатор");

		#ifdef WIN32    
    		set_icon_from_file(dirname+"\\kalkulator24.png");
		#else
    		set_icon_from_file(PACKAGE_DATA_DIR "/kalkulator/kalkulator24.png");		
    								// ako ne moze da pronadje filename, set_icon_from_file
		#endif						// bacice izuzetak Glib::FileError koji obradjujemo
	}
	catch(Glib::FileError e)
	{
    	std::cerr << e.what();
    	g_warning("Не могу да пронађем инсталирану иконицу прозора.\n");
	}
  
  // Korenski prozor
	Gtk::Window* Window = NULL;
	refGlade->get_widget("window", Window);	// kontrola koja se zove window iz Glade datoteke
	if(Window)
	{					// biva "ugoscena" u Gtk::Window* Window;

    	// Glavni kontejner u korenskom prozoru
    	refGlade->get_widget("toplevel", WidgetTopLevel);
    	if(WidgetTopLevel)
    	{
      		// Ukloni iz Glade prozora i ucitaj u tekuci:
      		WidgetTopLevel->reference(); 	// Uvecaj brojac referenci
      		Window->remove(); 		// Ukloni WidgetTopLevel (umanji brojac referenci)
      		delete Window;
      		add(*WidgetTopLevel);		// Dodaj WidgetTopLevel u Aplikacija (uvecaj brojac referenci)
      		WidgetTopLevel->unreference();    // Umanji brojac referenci
    	}
    	else
    		g_error("Глејд датотека сучеља није исправна.\n");

    	//Dugmad
    	refGlade->get_widget("izadji", ButtonIzadji);	// pribavlja pokazivac na dugme izadji i smesta ga u ButtonIzadji
    	ButtonIzadji->signal_clicked().connect( sigc::mem_fun(*this, &Aplikacija::hide) ); 
 							// Aplikacija::hide() nasledjeno je iz Gtk::Window
 							// i umanjuje broj referenci, efektivno pozivajuci destruktor Aplikacije
    	refGlade->get_widget("oprogramu", ButtonOProgramu);
    	ButtonOProgramu->signal_clicked().connect( sigc::mem_fun(*this, &Aplikacija::on_button_oprogramu) );

    	refGlade->get_widget("napred", ButtonNapred);
    	refGlade->get_widget("izracunaj", ButtonIzracunaj);
    	ButtonNapred->set_sensitive(false);    		// onemogucuje klik na dugme
    	ButtonIzracunaj->set_sensitive(false);
    	ButtonNapred->signal_clicked().connect( sigc::mem_fun(*this, &Aplikacija::on_button_napred) );
		ButtonIzracunaj->signal_clicked().connect( sigc::mem_fun(*this, &Aplikacija::on_button_izracunaj) );

    	refGlade->get_widget("prvi", EntryPrvi);
    	refGlade->get_widget("drugi", EntryDrugi);
    	EntryPrvi->signal_changed().connect( sigc::mem_fun(*this, &Aplikacija::on_unos_brojeva) );
    	EntryDrugi->signal_changed().connect( sigc::mem_fun(*this, &Aplikacija::on_unos_brojeva) );

    	refGlade->get_widget("zapis1", RadioZapis1);
    	refGlade->get_widget("zapis2", RadioZapis2);

    	refGlade->get_widget("rezultat", TextRezultat);
    	Pango::FontDescription font;
    	font.set_family("monospace");
		//font.set_size(14);
    	TextRezultat->modify_font(font);
  	}
  	else g_error("Глејд датотека сучеља није исправна.\n");
}


Aplikacija::~Aplikacija()
{
  std::cout << "~Aplikacija()" << std::endl;
}

/* 
  Pravi GTK MessageDialog sa tekstom o programu
  Pokrece dijalog i ceka da korisnik klikne na dugme da bi ga zatvorio
*/
void Aplikacija::on_button_oprogramu()
{
    Gtk::MessageDialog dialog(*this, "Калкулатор 1.0\n(c) 2006 Мирко Стојадиновић");
    dialog.run();
}

/*
  Ako je duzina oba operanda nula, konjukcija je nula i set_sensitive postavlja
  osetljivost na FALSE, inace prebacuje na TRUE
*/
void Aplikacija::on_unos_brojeva()
{
	ButtonIzracunaj->set_sensitive(
		EntryPrvi->get_text_length() && 
		EntryDrugi->get_text_length()
	);
}

void Aplikacija::on_button_izracunaj()
{

	brezultat.clear();

	std::string sPrvi = EntryPrvi->get_text();
	std::string sDrugi = EntryDrugi->get_text();
	BcdBroj *prvi, *drugi;
	try { RadioZapis1->get_active() ? prvi = new Broj8421(sPrvi) : prvi = new BrojVisak3(sPrvi); }
	catch(std::invalid_argument e)
	{
	    Gtk::MessageDialog dialog(*this, "Први операнд није исправан!", false, Gtk::MESSAGE_ERROR);
	    dialog.run();	   	
	    return;
	}
		
	try { RadioZapis1->get_active() ? drugi = new Broj8421(sDrugi) : drugi = new BrojVisak3(sDrugi); }
	catch(std::invalid_argument e)
	{
	    Gtk::MessageDialog dialog(*this, "Други операнд није исправан!", false, Gtk::MESSAGE_ERROR);
	    dialog.run();	   	
	    return;	   	
	}
	
	prvi->IzvrsiOperaciju(*drugi);
	ButtonNapred->set_sensitive(true);

	on_button_napred();
}

void Aplikacija::on_button_napred()
{
	TextRezultat->set_buffer(brezultat.PrikaziNaredni());
	ButtonNapred->set_sensitive(brezultat.PrikazanKorak() < brezultat.BrojKoraka());
}

Rezultat brezultat;


